@@ -13,24 +13,19 @@ from dbnomics_pptx_tools.repo import SeriesRepo
 logger = daiquiri.getLogger(__name__)
 
 
-def build_category_chart_data(chart_spec: ChartSpec, *, df: DataFrame) -> CategoryChartData:
-    chart_spec_series_ids = chart_spec.get_series_ids()
+def build_category_chart_data(chart_spec: ChartSpec, *, pivoted_df: DataFrame) -> CategoryChartData:
     chart_data = CategoryChartData()
 
-    pivoted_df = df.pivot(index="period", columns="series_id", values="value")
     chart_data.categories = cast(DatetimeIndex, pivoted_df.index).to_pydatetime()
 
-    for series_id in chart_spec_series_ids:
-        series_spec = chart_spec.find_series_spec(series_id)
-        if series_spec is None:
-            raise ValueError(f"Could not find spec for series {series_id!r}")
-        series_name = series_spec.name
+    for series_spec in chart_spec.series:
+        series_id = series_spec.id
         if series_id not in pivoted_df:
+            logger.warning("Could not find series %r in the pivoted DataFrame", series_id)
             continue
-        series = pivoted_df[series_id].replace({np.NaN: None})
-        for transformer in series_spec.transformers:
-            series = transformer(series_spec, series)
-        chart_data.add_series(series_name, series.values)
+        series = pivoted_df[series_id]
+        series = series.replace({np.NaN: None})
+        chart_data.add_series(series_spec.name, series.values)
 
     return chart_data
 
@@ -44,14 +39,15 @@ def filter_df_to_domain(df: DataFrame, *, max_datetime: datetime | None, min_dat
 
 
 def load_chart_df(chart_spec: ChartSpec, *, repo: SeriesRepo) -> DataFrame:
-    chart_spec_series_ids = chart_spec.get_series_ids()
-
-    def iter_domain_dfs() -> Iterator[DataFrame]:
-        for series_id in chart_spec_series_ids:
+    def iter_series_dfs() -> Iterator[DataFrame]:
+        for series_spec in chart_spec.series:
+            series_id = series_spec.id
             df = repo.load(series_id)
             if df.empty:
                 logger.warning("Series %r is empty", series_id)
                 continue
+            df["chart_series_name"] = series_spec.name
+            df["value"] = series_spec.apply_transformers(df["value"], series_id=series_id)
             yield df
 
-    return pd.concat(iter_domain_dfs())
+    return pd.concat(iter_series_dfs())
