@@ -53,20 +53,20 @@ def parse_range_option(expr: str) -> set[int]:
 def fetch(
     metadata_file: Path = typer.Argument(..., exists=True, readable=True),
     dbnomics_api_cache_dir: Path = typer.Option(DBNOMICS_API_CACHE_DIR_NAME),
-    force: bool = typer.Option(False, help="Fetch a series even if it is already stored in the cache."),
+    skip_existing: bool = typer.Option(False, help="Do not fetch the series that are already stored in the cache."),
 ):
     presentation_metadata = load_presentation_metadata(metadata_file)
     series_ids = sorted(presentation_metadata.get_slide_series_ids())
 
     cache = SeriesCache(cache_dir=dbnomics_api_cache_dir)
-    repo = SeriesRepo(auto_fetch=True, cache=cache, force=force)
+    repo = SeriesRepo(auto_fetch=True, cache=cache, force=not skip_existing)
 
     logger.debug("Fetching all the series needed for the presentation: %r...", series_ids)
 
     fetched_series_ids = []
     skipped_series_ids = []
     for series_id in series_ids:
-        if cache.has(series_id) and not force:
+        if cache.has(series_id) and skip_existing:
             logger.debug("Series %r is already stored in the cache, skipping it", series_id)
             skipped_series_ids.append(series_id)
             continue
