@@ -1,9 +1,11 @@
-from __future__ import annotations
-
+from collections import Counter, defaultdict
 from enum import Enum
 from typing import Callable, Literal, TypeAlias
 
+import daiquiri
+import isodate
 import pandas as pd
+from isodate import Duration
 from pydantic import BaseModel, Field, validator
 
 from dbnomics_pptx_tools.module_utils import parse_callable
@@ -11,14 +13,18 @@ from dbnomics_pptx_tools.module_utils import parse_callable
 __all__ = ["DataLabelPosition", "PresentationMetadata", "SlideMetadata"]
 
 
+logger = daiquiri.getLogger(__name__)
+
+
 class DataLabelPosition(Enum):
     LAST_POINT = "last_point"
 
 
 ChartName: TypeAlias = str
+DataLabelPositionT: TypeAlias = Literal["last_point"]
+Frequency: TypeAlias = Literal["annual", "monthly", "quarterly"]
 SeriesId: TypeAlias = str
 SeriesTransformer: TypeAlias = Callable[["SeriesSpec", pd.Series], pd.Series]
-SlideName: TypeAlias = str
 TableLocation: TypeAlias = str
 
 
@@ -37,11 +43,17 @@ class SeriesSpec(BaseModel):
             raise ValueError(f"The function referenced by {value!r} does not exist")
         return function
 
+    def apply_transformers(self, series: pd.Series, *, series_id: str) -> pd.Series:
+        for transformer in self.transformers:
+            logger.debug("Applying transformer %r to series %r", transformer.__qualname__, series_id)
+            series = transformer(self, series)
+        return series
+
     def has_tag(self, tag: str | None) -> bool:
         return tag == self.tag
 
 
-class TableSpec(BaseModel):
+class ShapeSpec(BaseModel):
     series: list[SeriesSpec]
 
     def find_series_id_by_name(self, series_name: str, *, tag: str | None = None) -> str | None:
@@ -63,14 +75,46 @@ class TableSpec(BaseModel):
     def get_series_ids(self) -> list[str]:
         return [series_id_or_spec.id for series_id_or_spec in self.series]
 
-
-DataLabelPositionT: TypeAlias = Literal["last_point"]
+    @validator("series")
+    def validate_series_name_is_unique(cls, value: list[SeriesSpec]):
+        names_by_tag = defaultdict(list)
+        for series_spec in value:
+            names_by_tag[series_spec.tag].append(series_spec.name)
+        for tag, names in names_by_tag.items():
+            counts = Counter(names)
+            duplicates = {name for name, count in counts.items() if count > 1}
+            if duplicates:
+                message = f"Series names must be unique, found those duplicates: {sorted(duplicates)!r}"
+                if tag is not None:
+                    message += f" (for tag {tag})"
+                raise ValueError(message)
+        return value
 
 
-class ChartSpec(TableSpec):
+class ChartSpec(ShapeSpec):
     data_labels: list[DataLabelPositionT] = Field(default_factory=list)
 
 
+class ColumnsSpec(BaseModel):
+    end_period_offset: Duration | None
+    frequency: Frequency
+    period_format: str
+    rolling_periods: bool
+
+    class Config:
+        arbitrary_types_allowed = True
+
+    @validator("end_period_offset", pre=True)
+    def parse_end_period_offset(cls, value: str):
+        if not isinstance(value, str):
+            raise ValueError(f"str expected, got {type(value)}")
+        return isodate.parse_duration(value)
+
+
+class TableSpec(ShapeSpec):
+    columns: ColumnsSpec | None
+
+
 class SlideMetadata(BaseModel):
     charts: dict[ChartName, ChartSpec] = Field(default_factory=dict)
     tables: dict[TableLocation, TableSpec] = Field(default_factory=dict)
@@ -85,7 +129,7 @@ class SlideMetadata(BaseModel):
 
 
 class PresentationMetadata(BaseModel):
-    slides: dict[SlideName, SlideMetadata]
+    slides: dict[str, SlideMetadata]
 
     def get_slide_series_ids(self) -> set[str]:
         result = set()
